@@ -58,6 +58,7 @@ pub trait HolochainP2pCellT {
     async fn publish(
         &self,
         request_validation_receipt: bool,
+        countersigning_session: bool,
         dht_hash: holo_hash::AnyDhtHash,
         ops: Vec<(holo_hash::DhtOpHash, holochain_types::dht_op::DhtOp)>,
         timeout_ms: Option<u64>,
@@ -111,6 +112,17 @@ pub trait HolochainP2pCellT {
         &self,
         dht_hash: holo_hash::AnyDhtHash,
     ) -> actor::HolochainP2pResult<bool>;
+
+    /// Response from an authority to agents that are
+    /// part of a session.
+    async fn countersigning_authority_response(
+        &self,
+        agents: Vec<AgentPubKey>,
+        response: Vec<SignedHeader>,
+    ) -> actor::HolochainP2pResult<()>;
+
+    /// New data has been integrated and is ready for gossiping.
+    async fn new_integrated_data(&self) -> actor::HolochainP2pResult<()>;
 }
 
 /// A wrapper around HolochainP2pSender that partially applies the dna_hash / agent_pub_key.
@@ -174,6 +186,7 @@ impl HolochainP2pCellT for HolochainP2pCell {
     async fn publish(
         &self,
         request_validation_receipt: bool,
+        countersigning_session: bool,
         dht_hash: holo_hash::AnyDhtHash,
         ops: Vec<(holo_hash::DhtOpHash, holochain_types::dht_op::DhtOp)>,
         timeout_ms: Option<u64>,
@@ -183,6 +196,7 @@ impl HolochainP2pCellT for HolochainP2pCell {
                 (*self.dna_hash).clone(),
                 (*self.from_agent).clone(),
                 request_validation_receipt,
+                countersigning_session,
                 dht_hash,
                 ops,
                 timeout_ms,
@@ -292,10 +306,36 @@ impl HolochainP2pCellT for HolochainP2pCell {
     /// Check if an agent is an authority for a hash.
     async fn authority_for_hash(
         &self,
-        _dht_hash: holo_hash::AnyDhtHash,
+        dht_hash: holo_hash::AnyDhtHash,
     ) -> actor::HolochainP2pResult<bool> {
-        // Currently everyone is an authority
-        Ok(true)
+        self.sender
+            .authority_for_hash(
+                (*self.dna_hash).clone(),
+                (*self.from_agent).clone(),
+                dht_hash,
+            )
+            .await
+    }
+
+    async fn countersigning_authority_response(
+        &self,
+        agents: Vec<AgentPubKey>,
+        response: Vec<SignedHeader>,
+    ) -> actor::HolochainP2pResult<()> {
+        self.sender
+            .countersigning_authority_response(
+                (*self.dna_hash).clone(),
+                (*self.from_agent).clone(),
+                agents,
+                response,
+            )
+            .await
+    }
+
+    async fn new_integrated_data(&self) -> actor::HolochainP2pResult<()> {
+        self.sender
+            .new_integrated_data((*self.dna_hash).clone())
+            .await
     }
 }
 
