@@ -219,6 +219,7 @@ ghost_actor::ghost_chan! {
             dna_hash: DnaHash,
             from_agent: AgentPubKey,
             request_validation_receipt: bool,
+            countersigning_session: bool,
             dht_hash: holo_hash::AnyDhtHash,
             ops: Vec<(holo_hash::DhtOpHash, holochain_types::dht_op::DhtOp)>,
             timeout_ms: Option<u64>,
@@ -262,6 +263,21 @@ ghost_actor::ghost_chan! {
 
         /// Send a validation receipt to a remote node.
         fn send_validation_receipt(dna_hash: DnaHash, to_agent: AgentPubKey, from_agent: AgentPubKey, receipt: SerializedBytes) -> ();
+
+        /// New data has been integrated and is ready for gossiping.
+        fn new_integrated_data(dna_hash: DnaHash) -> ();
+
+        /// Check if an agent is an authority for a hash.
+        fn authority_for_hash(dna_hash: DnaHash, from_agent: AgentPubKey, dht_hash: AnyDhtHash) -> bool;
+
+        /// Response from an authority to agents that are
+        /// part of a session.
+        fn countersigning_authority_response(
+            dna_hash: DnaHash,
+            from_agent: AgentPubKey,
+            agents: Vec<AgentPubKey>,
+            signed_headers: Vec<SignedHeader>,
+        ) -> ();
     }
 }
 
